package ca.uwaterloo.crysp.attackSS;

import java.util.ArrayList;

import android.util.Log;
import android.view.MotionEvent;

public class TouchFeatures {
	/*
	 * Swipe features used by SilentSense
	 * 0 - Direction (not a SilentSense feature; kept to keep track of swipe)
	 * 1,2- First touch position: x & y co-ords
	 * 3- Duration
	 * 4- Pressure
	 * 5- Vibration
	 * 6- Motion
	 */
	
	
	public static int featureSetSize = 7;
	static double [] fv = new double[featureSetSize];
	
	static ArrayList<TouchPoint> points = new ArrayList<TouchPoint>();

	public static ArrayList<TouchPoint> getTouchPoints() {
		return points;
	}
	
	public static double[] getFV() {
		double [] _fv = new double[featureSetSize];
		for (int i = 0; i < fv.length; i++)
			_fv[i] = fv[i];
		return _fv;
	}
	private static void populateFeatureVector() {
		/*0- Direction: pivot for different training models
		 * code: 0->Up; 1->Down; 2->Left; 3->Right*/
		float xDisp = points.get(0).x - points.get(points.size()-1).x;
		float yDisp = points.get(0).y - points.get(points.size()-1).y;
		if (Math.abs(xDisp) > Math.abs(yDisp))  /*L or R*/
			fv[0] = (xDisp > 0)? 2:3;
		else  /*U or D*/
			fv[0] = (yDisp > 0)? 0:1;
		
		/* 1,2- First touch position: x & y co-ords*/
		fv[1] = points.get(0).x;
		fv[2] = points.get(0).y;
		/* 3- Duration: time taken to swipe*/
		fv[3] = points.get(points.size()-1).tstamp - points.get(0).tstamp;
		/*4- Pressure*/
		fv[4] = points.get(0).pressure;
		/*5- Vibration*/
		fv[5] = SensorData.procSensorData(TracingActivity.gyr);
		/*6- Motion*/
		fv[6] = SensorData.procSensorData(TracingActivity.acc);
	}
	public static boolean procEvent(MotionEvent ev) {
    	switch (ev.getAction()) {
    	case MotionEvent.ACTION_DOWN:  /* primary pointer */
		case MotionEvent.ACTION_POINTER_DOWN: /* any subsequent pointer */
			points.clear();//clear points now; must've been fetched by now
			TracingActivity.setSwiping(true);
			TracingActivity.sensorReadings.clear();
			TracingActivity.acc.clear();
			TracingActivity.gyr.clear();
			return false;
		case MotionEvent.ACTION_MOVE: /* any number of pointers move */
			for (int hIndx = 0; hIndx < ev.getHistorySize(); hIndx++) {
				for (int pIndex = 0; pIndex < ev.getPointerCount(); 
						pIndex++) {
					TouchPoint tp = new TouchPoint();
					tp.x = ev.getHistoricalX(pIndex, hIndx);
					tp.y = ev.getHistoricalY(pIndex, hIndx);
					tp.area = ev.getHistoricalSize(pIndex, hIndx);
					tp.tstamp = ev.getHistoricalEventTime(hIndx);
					tp.pressure = ev.getHistoricalPressure(pIndex, hIndx); 
					points.add(tp);
				}
			}
			for (int pIndex = 0; pIndex < ev.getPointerCount(); 
					pIndex++) {
				TouchPoint tp = new TouchPoint();
				tp.x = ev.getX(pIndex);
				tp.y = ev.getY(pIndex);
				tp.area = ev.getSize(pIndex);
				tp.tstamp = ev.getEventTime();
				tp.pressure = ev.getPressure();
				points.add(tp);
			}
			return false;
		case MotionEvent.ACTION_POINTER_UP: /* all pointers are up */
		case MotionEvent.ACTION_UP: 
		case MotionEvent.ACTION_CANCEL:
			TracingActivity.setSwiping(false);
			if (points.size() > 5) {
				populateFeatureVector();
				return true;
			}
			//points.clear();
			return false;
    	}
    	return false;
	}

	static double[] TouchPointsToFV(ArrayList<TouchPoint> _points) {
		double [] _fv = new double[featureSetSize];
		/*0- Direction: pivot for different training models
		 * code: 0->Up; 1->Down; 2->Left; 3->Right*/
		float xDisp = _points.get(0).x - _points.get(_points.size()-1).x;
		float yDisp = _points.get(0).y - _points.get(_points.size()-1).y;
		if (Math.abs(xDisp) > Math.abs(yDisp))  /*L or R*/
			_fv[0] = (xDisp > 0)? 2:3;
		else  /*U or D*/
			_fv[0] = (yDisp > 0)? 0:1;
		
		/* 1,2- First touch position: x & y co-ords*/
		_fv[1] = _points.get(0).x;
		_fv[2] = _points.get(0).y;
		/* 3- Duration: time taken to swipe*/
		_fv[3] = _points.get(_points.size()-1).tstamp - _points.get(0).tstamp;
		/*4- Pressure*/
		_fv[4] = _points.get(0).pressure;
		//XXX: populate vibration and motion features
		return _fv;
	}
}
